from ptee.instance import Instance
import argparse
example_str = """
Examples:

1. save to path

   ptee --path=/tmp/log/one.log

2. set backup count

   ptee --path=/tmp/log/one.log --backup=7

3. set file size

   ptee --path=/tmp/log/one.log --max_byte=50m

4. with debug

   ptee --path=/tmp/log/one.log --debug=1
"""

import os
_dir = os.path.dirname(os.path.abspath(__file__))
init_path = os.path.join(_dir, '__init__.py')


def read_version():
    d = {}
    code = open(init_path).read()
    code = compile(code, '<string>', 'exec', dont_inherit=True)
    exec(code, d, d)
    return d['__version__']


parser = argparse.ArgumentParser('ptee',
                                 description='ptee ',
                                 formatter_class=argparse.RawDescriptionHelpFormatter,
                                 epilog=example_str)
parser.add_argument('--path', dest='path',
                    help='save path', required=True)
parser.add_argument('--backup', dest='backup',
                    default=7, help='backup count')
# parser.add_argument('--encode', dest='encode', default='utf-8')
# parser.add_argument('--line-filter', dest='line_filter', help='regex for line filter')
parser.add_argument("--max_byte", dest="max_byte", default="50m", help="ex 50m, 20k, 5G")
parser.add_argument("--debug", dest="debug", default="0", help="show debug message")
parser.add_argument("--version", action="version", version=read_version())


def main(argv=None):
    if argv is not None:
        convert_args = parser.parse_args(argv)
    else:
        convert_args = parser.parse_args()

    # generated_by_dict_unpack: convert_args
    path, backup, max_byte = convert_args.path, convert_args.backup, convert_args.max_byte
    debug = int(convert_args.debug)
    backup = int(backup)

    ins = Instance(path, backup, max_byte, debug)
    ins.loop()



import os
import time
import sys


class Instance(object):

    def __init__(self, path, backup, max_byte, debug, dir_mod=0o777):
        self.path = path
        self.backup = backup
        self.max_byte = max_byte
        self.debug = debug
        self.dir_mod = dir_mod

    def bytestr_to_size(self, byte_str):
        unit_map = {
            "k": 1024,
            "m": 1024 * 1024,
            "g": 1024 * 1024 * 1024
        }
        last_elment = byte_str[-1]
        unit = 1

        if last_elment in unit_map:
            byte_str = byte_str[:-1]
            unit = unit_map[last_elment]
        return int(byte_str) * unit

    def size_pretty(self, size):
        unit_list = ['k', 'm', 'g']
        unit = ''
        unit_index = -1
        while size > 1024:
            size = size // 1024
            unit_index += 1
            unit = unit_list[unit_index]
        return '{}{}'.format(size, unit)

    def create_path_dir_if_not_exists(self):
        dir_name = os.path.dirname(self.path)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name, self.dir_mod)

    def get_path_size(self, path):
        if not os.path.exists(path):
            return 0
        st = os.stat(path)
        return st.st_size

    def loop(self, interval=0.1):
        self.create_path_dir_if_not_exists()
        max_size_int = self.bytestr_to_size(self.max_byte)
        path_file = open(self.path, 'a', encoding='utf-8')
        # while 1:
            # read_str = sys.stdin.read()
        for read_str in sys.stdin:
            if read_str:
                sys.stdout.write(read_str)
                sys.stdout.flush()
                path_size = self.get_path_size(self.path)

                self.log_debug_message("max_size_pretty:{}, path_size_pretty:{}".format(self.max_byte, self.size_pretty(path_size)))
                if path_size > max_size_int:
                    self.backup_path(self.path)
                path_file.close()
                path_file = open(self.path, 'a', encoding='utf-8')
                path_file.write(read_str)
            # time.sleep(interval)

    def backup_path(self, path):
        path_list = [
            path,

        ]
        for i in range(1, self.backup + 1):
            path_list.append(
                "{}.{}".format(path, i)
            )
        n = len(path_list)
        for i in range(n - 2, -1, -1):
            src_path = path_list[i]
            dest_path = path_list[i + 1]
            message = "try backup path {}->{}".format(src_path, dest_path)
            self.log_debug_message(message)
            if os.path.exists(src_path):
                os.rename(src_path, dest_path)

    def log_debug_message(self, message):
        if self.debug:
            if not message.startswith(os.linesep):
                message = os.linesep + message
            if not message.endswith(os.linesep):
                message += os.linesep
            sys.stdout.write(message)
